@@ -21,376 +21,6 @@
 
 #include "_arraystruct.h"
 
-/*
- * PRNG support.
- */
-
-/* Magic Mersenne Twister constants */
-#define MT_N 624
-#define MT_M 397
-#define MT_MATRIX_A 0x9908b0dfU
-#define MT_UPPER_MASK 0x80000000U
-#define MT_LOWER_MASK 0x7fffffffU
-
-/* unsigned int is sufficient on modern machines as we only need 32 bits */
-typedef struct {
-    int index;
-    unsigned int mt[MT_N];
-    int has_gauss;
-    double gauss;
-} rnd_state_t;
-
-NUMBA_EXPORT_DATA(rnd_state_t) numba_py_random_state;
-NUMBA_EXPORT_DATA(rnd_state_t) numba_np_random_state;
-
-/* Some code portions below from CPython's _randommodule.c, some others
-   from Numpy's and Jean-Sebastien Roy's randomkit.c. */
-
-NUMBA_EXPORT_FUNC(void)
-numba_rnd_shuffle(rnd_state_t *state)
-{
-    int i;
-    unsigned int y;
-
-    for (i = 0; i < MT_N - MT_M; i++) {
-        y = (state->mt[i] & MT_UPPER_MASK) | (state->mt[i+1] & MT_LOWER_MASK);
-        state->mt[i] = state->mt[i+MT_M] ^ (y >> 1) ^
-                       (-(int) (y & 1) & MT_MATRIX_A);
-    }
-    for (; i < MT_N - 1; i++) {
-        y = (state->mt[i] & MT_UPPER_MASK) | (state->mt[i+1] & MT_LOWER_MASK);
-        state->mt[i] = state->mt[i+(MT_M-MT_N)] ^ (y >> 1) ^
-                       (-(int) (y & 1) & MT_MATRIX_A);
-    }
-    y = (state->mt[MT_N - 1] & MT_UPPER_MASK) | (state->mt[0] & MT_LOWER_MASK);
-    state->mt[MT_N - 1] = state->mt[MT_M - 1] ^ (y >> 1) ^
-                          (-(int) (y & 1) & MT_MATRIX_A);
-}
-
-/* Initialize mt[] with an integer seed */
-NUMBA_EXPORT_FUNC(void)
-numba_rnd_init(rnd_state_t *state, unsigned int seed)
-{
-    unsigned int pos;
-    seed &= 0xffffffffU;
-
-    /* Knuth's PRNG as used in the Mersenne Twister reference implementation */
-    for (pos = 0; pos < MT_N; pos++) {
-        state->mt[pos] = seed;
-        seed = (1812433253U * (seed ^ (seed >> 30)) + pos + 1) & 0xffffffffU;
-    }
-    state->index = MT_N;
-    state->has_gauss = 0;
-    state->gauss = 0.0;
-}
-
-/* Perturb mt[] with a key array */
-static void
-rnd_init_by_array(rnd_state_t *state, unsigned int init_key[], size_t key_length)
-{
-    size_t i, j, k;
-    unsigned int *mt = state->mt;
-
-    numba_rnd_init(state, 19650218U);
-    i = 1; j = 0;
-    k = (MT_N > key_length ? MT_N : key_length);
-    for (; k; k--) {
-        mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1664525U))
-                 + init_key[j] + (unsigned int) j; /* non linear */
-        mt[i] &= 0xffffffffU;
-        i++; j++;
-        if (i >= MT_N) { mt[0] = mt[MT_N - 1]; i = 1; }
-        if (j >= key_length) j = 0;
-    }
-    for (k = MT_N - 1; k; k--) {
-        mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1566083941U))
-                 - (unsigned int) i; /* non linear */
-        mt[i] &= 0xffffffffU;
-        i++;
-        if (i >= MT_N) { mt[0] = mt[MT_N - 1]; i=1; }
-    }
-
-    mt[0] = 0x80000000U; /* MSB is 1; ensuring non-zero initial array */
-    state->index = MT_N;
-    state->has_gauss = 0;
-    state->gauss = 0.0;
-}
-
-/* Random-initialize the given state (for use at startup) */
-NUMBA_EXPORT_FUNC(int)
-_numba_rnd_random_seed(rnd_state_t *state)
-{
-    PyObject *timemod, *timeobj;
-    double timeval;
-    Py_uintptr_t seed;
-    unsigned int seed32;
-    void *dummy;
-
-    /* XXX we could get a seed using _PyOS_URandom() instead */
-
-    timemod = PyImport_ImportModuleNoBlock("time");
-    if (!timemod)
-        return -1;
-    timeobj = PyObject_CallMethod(timemod, "time", NULL);
-    Py_DECREF(timemod);
-    timeval = PyFloat_AsDouble(timeobj);
-    Py_DECREF(timeobj);
-    if (timeval == -1 && PyErr_Occurred())
-        return -1;
-    /* Mix in seconds and nanoseconds */
-    seed = (Py_uintptr_t) timeval ^ (Py_uintptr_t) (timeval * 1e9);
-#ifndef _WIN32
-    seed ^= getpid();
-#endif
-    /* Address space randomization bits: mix in various pointers. */
-    seed ^= (Py_uintptr_t) &timemod;
-    seed += (Py_uintptr_t) &PyObject_CallMethod >> 3;
-    seed += (Py_uintptr_t) &rnd_init_by_array;
-    dummy = malloc(1);
-    free(dummy);
-    seed += (Py_uintptr_t) &dummy;
-
-    /* Reduce to 32 bits for Mersenne Twisted seeding */
-    seed32 = (unsigned int) (seed ^ (seed >> 16));
-    numba_rnd_init(state, seed32);
-    return 0;
-}
-
-/* Python-exposed helpers for state management */
-static int
-rnd_state_converter(PyObject *obj, rnd_state_t **state)
-{
-    *state = (rnd_state_t *) PyLong_AsVoidPtr(obj);
-    return (*state != NULL || !PyErr_Occurred());
-}
-
-NUMBA_EXPORT_FUNC(PyObject *)
-_numba_rnd_shuffle(PyObject *self, PyObject *arg)
-{
-    rnd_state_t *state;
-    if (!rnd_state_converter(arg, &state))
-        return NULL;
-    numba_rnd_shuffle(state);
-    Py_RETURN_NONE;
-}
-
-NUMBA_EXPORT_FUNC(PyObject *)
-_numba_rnd_set_state(PyObject *self, PyObject *args)
-{
-    int i, index;
-    rnd_state_t *state;
-    PyObject *tuplearg, *intlist;
-
-    if (!PyArg_ParseTuple(args, "O&O!:rnd_set_state",
-                          rnd_state_converter, &state,
-                          &PyTuple_Type, &tuplearg))
-        return NULL;
-    if (!PyArg_ParseTuple(tuplearg, "iO!", &index, &PyList_Type, &intlist))
-        return NULL;
-    if (PyList_GET_SIZE(intlist) != MT_N) {
-        PyErr_SetString(PyExc_ValueError, "list object has wrong size");
-        return NULL;
-    }
-    state->index = index;
-    for (i = 0; i < MT_N; i++) {
-        PyObject *v = PyList_GET_ITEM(intlist, i);
-        unsigned long x = PyLong_AsUnsignedLong(v);
-        if (x == (unsigned long) -1 && PyErr_Occurred())
-            return NULL;
-        state->mt[i] = (unsigned int) x;
-    }
-    state->has_gauss = 0;
-    state->gauss = 0.0;
-    Py_RETURN_NONE;
-}
-
-NUMBA_EXPORT_FUNC(PyObject *)
-_numba_rnd_get_state(PyObject *self, PyObject *arg)
-{
-    PyObject *intlist;
-    int i;
-    rnd_state_t *state;
-    if (!rnd_state_converter(arg, &state))
-        return NULL;
-
-    intlist = PyList_New(MT_N);
-    if (intlist == NULL)
-        return NULL;
-    for (i = 0; i < MT_N; i++) {
-        PyObject *v = PyLong_FromUnsignedLong(state->mt[i]);
-        if (v == NULL) {
-            Py_DECREF(intlist);
-            return NULL;
-        }
-        PyList_SET_ITEM(intlist, i, v);
-    }
-    return Py_BuildValue("iN", state->index, intlist);
-}
-
-NUMBA_EXPORT_FUNC(PyObject *)
-_numba_rnd_seed_with_urandom(PyObject *self, PyObject *args)
-{
-    rnd_state_t *state;
-    Py_buffer buf;
-    unsigned int *keys;
-    unsigned char *bytes;
-    size_t i, nkeys;
-
-    if (!PyArg_ParseTuple(args, "O&s*:rnd_seed",
-                          rnd_state_converter, &state, &buf)) {
-        return NULL;
-    }
-    /* Make a copy to avoid alignment issues */
-    nkeys = buf.len / sizeof(unsigned int);
-    keys = (unsigned int *) PyMem_Malloc(nkeys * sizeof(unsigned int));
-    if (keys == NULL) {
-        PyBuffer_Release(&buf);
-        return NULL;
-    }
-    bytes = (unsigned char *) buf.buf;
-    for (i = 0; i < nkeys; i++, bytes += 4) {
-        keys[i] = (bytes[3] << 24) + (bytes[2] << 16) +
-                  (bytes[1] << 8) + (bytes[0] << 0);
-    }
-    PyBuffer_Release(&buf);
-    rnd_init_by_array(state, keys, nkeys);
-    PyMem_Free(keys);
-    Py_RETURN_NONE;
-}
-
-NUMBA_EXPORT_FUNC(PyObject *)
-_numba_rnd_seed(PyObject *self, PyObject *args)
-{
-    unsigned int seed;
-    rnd_state_t *state;
-
-    if (!PyArg_ParseTuple(args, "O&I:rnd_seed",
-                          rnd_state_converter, &state, &seed)) {
-        PyErr_Clear();
-        return _numba_rnd_seed_with_urandom(self, args);
-    }
-    numba_rnd_init(state, seed);
-    Py_RETURN_NONE;
-}
-
-/* Random distribution helpers.
- * Most code straight from Numpy's distributions.c. */
-
-#ifndef M_PI
-#define M_PI 3.14159265358979323846264338328
-#endif
-
-NUMBA_EXPORT_FUNC(unsigned int)
-get_next_int32(rnd_state_t *state)
-{
-    unsigned int y;
-
-    if (state->index == MT_N) {
-        numba_rnd_shuffle(state);
-        state->index = 0;
-    }
-    y = state->mt[state->index++];
-    /* Tempering */
-    y ^= (y >> 11);
-    y ^= (y << 7) & 0x9d2c5680U;
-    y ^= (y << 15) & 0xefc60000U;
-    y ^= (y >> 18);
-    return y;
-}
-
-NUMBA_EXPORT_FUNC(double)
-get_next_double(rnd_state_t *state)
-{
-    double a = get_next_int32(state) >> 5;
-    double b = get_next_int32(state) >> 6;
-    return (a * 67108864.0 + b) / 9007199254740992.0;
-}
-
-NUMBA_EXPORT_FUNC(double)
-loggam(double x)
-{
-    double x0, x2, xp, gl, gl0;
-    long k, n;
-
-    static double a[10] = {8.333333333333333e-02,-2.777777777777778e-03,
-         7.936507936507937e-04,-5.952380952380952e-04,
-         8.417508417508418e-04,-1.917526917526918e-03,
-         6.410256410256410e-03,-2.955065359477124e-02,
-         1.796443723688307e-01,-1.39243221690590e+00};
-    x0 = x;
-    n = 0;
-    if ((x == 1.0) || (x == 2.0))
-    {
-        return 0.0;
-    }
-    else if (x <= 7.0)
-    {
-        n = (long)(7 - x);
-        x0 = x + n;
-    }
-    x2 = 1.0/(x0*x0);
-    xp = 2*M_PI;
-    gl0 = a[9];
-    for (k=8; k>=0; k--)
-    {
-        gl0 *= x2;
-        gl0 += a[k];
-    }
-    gl = gl0/x0 + 0.5*log(xp) + (x0-0.5)*log(x0) - x0;
-    if (x <= 7.0)
-    {
-        for (k=1; k<=n; k++)
-        {
-            gl -= log(x0-1.0);
-            x0 -= 1.0;
-        }
-    }
-    return gl;
-}
-
-
-NUMBA_EXPORT_FUNC(int64_t)
-numba_poisson_ptrs(rnd_state_t *state, double lam)
-{
-    /* This method is invoked only if the parameter lambda of this
-     * distribution is big enough ( >= 10 ). The algorithm used is
-     * described in "Hörmann, W. 1992. 'The Transformed Rejection
-     * Method for Generating Poisson Random Variables'.
-     * The implementation comes straight from Numpy.
-     */
-    int64_t k;
-    double U, V, slam, loglam, a, b, invalpha, vr, us;
-
-    slam = sqrt(lam);
-    loglam = log(lam);
-    b = 0.931 + 2.53*slam;
-    a = -0.059 + 0.02483*b;
-    invalpha = 1.1239 + 1.1328/(b-3.4);
-    vr = 0.9277 - 3.6224/(b-2);
-
-    while (1)
-    {
-        U = get_next_double(state) - 0.5;
-        V = get_next_double(state);
-        us = 0.5 - fabs(U);
-        k = (int64_t) floor((2*a/us + b)*U + lam + 0.43);
-        if ((us >= 0.07) && (V <= vr))
-        {
-            return k;
-        }
-        if ((k < 0) ||
-            ((us < 0.013) && (V > us)))
-        {
-            continue;
-        }
-        if ((log(V) + log(invalpha) - log(a/(us*us)+b)) <=
-            (-lam + (double) k*loglam - loggam((double) k+1)))
-        {
-            return k;
-        }
-    }
-}
-
 /*
  * Other helpers.
  */
@@ -752,6 +382,26 @@ numba_ndarray_new(int nd,
     return ndary;
 }
 
+
+/*
+ * Handle reshaping of zero-sized array.
+ * See numba_attempt_nocopy_reshape() below.
+ */
+static int
+nocopy_empty_reshape(npy_intp nd, const npy_intp *dims, const npy_intp *strides,
+                     npy_intp newnd, const npy_intp *newdims,
+                     npy_intp *newstrides, npy_intp itemsize,
+                     int is_f_order)
+{
+    int i;
+    /* Just make the strides vaguely reasonable
+     * (they can have any value in theory).
+     */
+    for (i = 0; i < newnd; i++)
+        newstrides[i] = itemsize;
+    return 1;  /* reshape successful */
+}
+
 /*
  * Straight from Numpy's _attempt_nocopy_reshape()
  * (np/core/src/multiarray/shape.c).
@@ -764,6 +414,7 @@ numba_ndarray_new(int nd,
  * If no copy is needed, returns 1 and fills `npy_intp *newstrides`
  *     with appropriate strides
  */
+
 NUMBA_EXPORT_FUNC(int)
 numba_attempt_nocopy_reshape(npy_intp nd, const npy_intp *dims, const npy_intp *strides,
                              npy_intp newnd, const npy_intp *newdims,
@@ -803,8 +454,10 @@ numba_attempt_nocopy_reshape(npy_intp nd, const npy_intp *dims, const npy_intp *
     }
 
     if (np == 0) {
-        /* the current code does not handle 0-sized arrays, so give up */
-        return 0;
+        /* the Numpy code does not handle 0-sized arrays */
+        return nocopy_empty_reshape(nd, dims, strides,
+                                    newnd, newdims, newstrides,
+                                    itemsize, is_f_order);
     }
 
     /* oi to oj and ni to nj give the axis ranges currently worked with */
@@ -1294,3 +947,9 @@ numba_unpickle(const char *data, int n)
  */
 
 #include "_lapack.c"
+
+/*
+ * PRNG support
+ */
+
+#include "_random.c"
