@@ -7,7 +7,6 @@ from __future__ import print_function, division, absolute_import
 import collections
 from contextlib import contextmanager
 import functools
-import re
 
 from llvmlite import ir
 
@@ -560,18 +559,31 @@ def for_range_slice_generic(builder, start, stop, step):
 
 
 @contextmanager
-def loop_nest(builder, shape, intp):
+def loop_nest(builder, shape, intp, order='C'):
     """
     Generate a loop nest walking a N-dimensional array.
-    Yields a tuple of N indices for use in the inner loop body.
+    Yields a tuple of N indices for use in the inner loop body,
+    iterating over the *shape* space.
+
+    If *order* is 'C' (the default), indices are incremented inside-out
+    (i.e. (0,0), (0,1), (0,2), (1,0) etc.).
+    If *order* is 'F', they are incremented outside-in
+    (i.e. (0,0), (1,0), (2,0), (0,1) etc.).
+    This has performance implications when walking an array as it impacts
+    the spatial locality of memory accesses.
     """
+    assert order in 'CF'
     if not shape:
         # 0-d array
         yield ()
     else:
-        with _loop_nest(builder, shape, intp) as indices:
+        if order == 'F':
+            _swap = lambda x: x[::-1]
+        else:
+            _swap = lambda x: x
+        with _loop_nest(builder, _swap(shape), intp) as indices:
             assert len(indices) == len(shape)
-            yield indices
+            yield _swap(indices)
 
 
 @contextmanager
@@ -586,8 +598,9 @@ def _loop_nest(builder, shape, intp):
 
 def pack_array(builder, values, ty=None):
     """
-    Pack an array of values.  *ty* should be given if the array may be empty,
-    in which case the type can't be inferred from the values.
+    Pack a sequence of values in a LLVM array.  *ty* should be given
+    if the array may be empty, in which case the type can't be inferred
+    from the values.
     """
     n = len(values)
     if ty is None:
@@ -967,3 +980,19 @@ def printf(builder, format, *args):
     # Call
     ptr_fmt = builder.bitcast(global_fmt, cstring)
     return builder.call(fn, [ptr_fmt] + list(args))
+
+
+if utils.PY3:
+    def normalize_ir_text(text):
+        """
+        Normalize the given string to latin1 compatible encoding that is suitable
+        for use in LLVM IR.
+        """
+        # Just re-encoding to latin1 is enough
+        return text.encode('utf8').decode('latin1')
+else:
+    def normalize_ir_text(text):
+        """
+        No-op for python2. Assume there won't be unicode names.
+        """
+        return text
