@@ -4,11 +4,14 @@ on the object.  If it exists and evaluate to True, it must define shape,
 strides, dtype and size attributes similar to a NumPy ndarray.
 """
 from __future__ import print_function, absolute_import, division
+
 import warnings
 import math
 import copy
 from ctypes import c_void_p
+
 import numpy as np
+
 from . import driver as _driver
 from . import devices
 from numba import dummyarray, types, numpy_support
@@ -63,7 +66,7 @@ class DeviceNDArrayBase(object):
         strides
             array strides.
         dtype
-            data type as numpy.dtype.
+            data type as np.dtype.
         stream
             cuda stream.
         writeback
