@@ -15,7 +15,7 @@ from numba import _dispatcher, compiler, utils, types, config, errors
 from numba.typeconv.rules import default_type_manager
 from numba import sigutils, serialize, typing
 from numba.typing.templates import fold_arguments
-from numba.typing.typeof import typeof, Purpose
+from numba.typing.typeof import Purpose, typeof, typeof_impl
 from numba.bytecode import get_code_object
 from numba.six import create_bound_method, next
 from .caching import NullCache, FunctionCache
@@ -32,6 +32,10 @@ class OmittedArg(object):
     def __repr__(self):
         return "omitted arg(%r)" % (self.value,)
 
+    @property
+    def _numba_type_(self):
+        return types.Omitted(self.value)
+
 
 class _FunctionCompiler(object):
 
@@ -150,8 +154,8 @@ class _DispatcherBase(_dispatcher.Dispatcher):
         self.__code__ = self.func_code
 
         argnames = tuple(pysig.parameters)
-        defargs = tuple(OmittedArg(val)
-                        for val in (self.py_func.__defaults__ or ()))
+        default_values = self.py_func.__defaults__ or ()
+        defargs = tuple(OmittedArg(val) for val in default_values)
         try:
             lastarg = list(pysig.parameters.values())[-1]
         except IndexError:
@@ -213,7 +217,7 @@ class _DispatcherBase(_dispatcher.Dispatcher):
         """Disable the compilation of new signatures at call time.
         """
         # If disabling compilation then there must be at least one signature
-        assert val or len(self.signatures) > 0
+        assert (not val) or len(self.signatures) > 0
         self._can_compile = not val
 
     def add_overload(self, cres):
@@ -562,22 +566,25 @@ class LiftedLoop(_DispatcherBase):
     """
     _fold_args = False
 
-    def __init__(self, bytecode, typingctx, targetctx, locals, flags):
+    def __init__(self, interp, typingctx, targetctx, flags, locals):
+        self.bytecode = interp.bytecode
+        self.interp = interp
+        self.lifted_from = None
+
         self.typingctx = typingctx
         self.targetctx = targetctx
-
-        _DispatcherBase.__init__(self, bytecode.arg_count, bytecode.func,
-                                 bytecode.pysig)
-
-        self.locals = locals
         self.flags = flags
-        self.bytecode = bytecode
-        self.lifted_from = None
+        self.locals = locals
+
+        _DispatcherBase.__init__(self, self.bytecode.arg_count,
+                                 self.bytecode.func, self.bytecode.pysig)
 
     def get_source_location(self):
         """Return the starting line number of the loop.
         """
-        return next(iter(self.bytecode)).lineno
+        firstblock = self.interp.blocks[min(self.interp.blocks)]
+        inst = firstblock.body[0]
+        return inst.loc.line
 
     def compile(self, sig):
         with self._compile_lock:
@@ -592,14 +599,13 @@ class LiftedLoop(_DispatcherBase):
                 return existing.entry_point
 
             assert not flags.enable_looplift, "Enable looplift flags is on"
-            cres = compiler.compile_bytecode(typingctx=self.typingctx,
-                                             targetctx=self.targetctx,
-                                             bc=self.bytecode,
-                                             args=args,
-                                             return_type=return_type,
-                                             flags=flags,
-                                             locals=self.locals,
-                                             lifted=(), lifted_from=self.lifted_from)
+            cres = compiler.compile_ir(typingctx=self.typingctx,
+                                       targetctx=self.targetctx,
+                                       interp=self.interp,
+                                       args=args, return_type=return_type,
+                                       flags=flags, locals=self.locals,
+                                       lifted=(),
+                                       lifted_from=self.lifted_from)
 
             # Check typing error if object mode is used
             if cres.typing_error is not None and not flags.enable_pyobject:
@@ -610,4 +616,6 @@ class LiftedLoop(_DispatcherBase):
 
 
 # Initialize typeof machinery
-_dispatcher.typeof_init(dict((str(t), t._code) for t in types.number_domain))
+_dispatcher.typeof_init(
+    OmittedArg,
+    dict((str(t), t._code) for t in types.number_domain))
