@@ -265,6 +265,24 @@ class UFuncLoopSpec(collections.namedtuple('_UFuncLoopSpec',
         return [as_dtype(x) for x in self.outputs]
 
 
+def ufunc_can_cast(from_, to, has_mixed_inputs, casting='safe'):
+    """
+    A variant of np.can_cast() that can allow casting any integer to
+    any real or complex type, in case the operation has mixed-kind
+    inputs.
+
+    For example we want `np.power(float32, int32)` to be computed using
+    SP arithmetic and return `float32`.
+    However, `np.sqrt(int32)` should use DP arithmetic and return `float64`.
+    """
+    from_ = np.dtype(from_)
+    to = np.dtype(to)
+    if has_mixed_inputs and from_.kind in 'iu' and to.kind in 'cf':
+        # Decide that all integers can cast to any real or complex type.
+        return True
+    return np.can_cast(from_, to, casting)
+
+
 def ufunc_find_matching_loop(ufunc, arg_types):
     """Find the appropriate loop to be used for a ufunc based on the types
     of the operands
@@ -290,6 +308,11 @@ def ufunc_find_matching_loop(ufunc, arg_types):
     except NotImplementedError:
         return None
 
+    # Whether the inputs are mixed integer / floating-point
+    has_mixed_inputs = (
+        any(dt.kind in 'iu' for dt in np_input_types) and
+        any(dt.kind in 'cf' for dt in np_input_types))
+
     def choose_types(numba_types, ufunc_letters):
         """
         Return a list of Numba types representing *ufunc_letters*,
@@ -327,22 +350,30 @@ def ufunc_find_matching_loop(ufunc, arg_types):
                 if outer.char != inner:
                     found = False
                     break
-            elif not np.can_cast(outer.char, inner, 'safe'):
+            elif not ufunc_can_cast(outer.char, inner,
+                                    has_mixed_inputs, 'safe'):
                 found = False
                 break
         if found and strict_ufunc_typing:
             # Can we cast the inner result to the outer result type?
             for outer, inner in zip(np_output_types, ufunc_outputs):
                 if (outer.char not in 'mM' and not
-                    np.can_cast(inner, outer.char, 'same_kind')):
+                    ufunc_can_cast(inner, outer.char,
+                                   has_mixed_inputs, 'same_kind')):
                     found = False
                     break
         if found:
             # Found: determine the Numba types for the loop's inputs and
             # outputs.
-            inputs = choose_types(input_types, ufunc_inputs)
-            outputs = choose_types(output_types, ufunc_outputs)
-            return UFuncLoopSpec(inputs, outputs, candidate)
+            try:
+                inputs = choose_types(input_types, ufunc_inputs)
+                outputs = choose_types(output_types, ufunc_outputs)
+            except NotImplementedError:
+                # One of the selected dtypes isn't supported by Numba
+                # (e.g. float16), try other candidates
+                continue
+            else:
+                return UFuncLoopSpec(inputs, outputs, candidate)
 
     return None
 
@@ -356,7 +387,10 @@ def from_struct_dtype(dtype):
         raise TypeError("Do not support dtype containing object")
 
     fields = {}
-    for name, (elemdtype, offset) in dtype.fields.items():
+
+    for name, info in dtype.fields.items():
+        # *info* may have 3 element if it has a "title", which can be ignored
+        [elemdtype, offset] = info[:2]
         fields[name] = from_dtype(elemdtype), offset
 
     # Note: dtype.alignment is not consistent.
