@@ -2,6 +2,7 @@ from __future__ import print_function, division, absolute_import
 
 import errno
 import imp
+import multiprocessing
 import os
 import shutil
 import stat
@@ -849,7 +850,8 @@ class TestCache(BaseCacheTest):
         except subprocess.CalledProcessError as e:
             self.skipTest("ipython not available: return code %d"
                           % e.returncode)
-        print("ipython version:", ver.strip().decode())
+        ver = ver.strip().decode()
+        print("ipython version:", ver)
         # Create test input
         inputfn = os.path.join(self.tempdir, "ipython_cache_usecase.txt")
         with open(inputfn, "w") as f:
@@ -859,9 +861,9 @@ class TestCache(BaseCacheTest):
 
                 from numba import jit
 
-                @jit(cache=True)
-                def f():
-                    return 42
+                # IPython 5 does not support multiline input if stdin isn't
+                # a tty (https://github.com/ipython/ipython/issues/9752)
+                f = jit(cache=True)(lambda: 42)
 
                 res = f()
                 # IPython writes on stdout, so use stderr instead
@@ -894,5 +896,30 @@ class TestCache(BaseCacheTest):
         self.assertEqual(err.strip(), "cache hits = 1")
 
 
+class TestMultiprocessCache(BaseCacheTest):
+
+    # Nested multiprocessing.Pool raises AssertionError:
+    # "daemonic processes are not allowed to have children"
+    _numba_parallel_test_ = False
+
+    here = os.path.dirname(__file__)
+    usecases_file = os.path.join(here, "cache_usecases.py")
+    modname = "dispatcher_caching_test_fodder"
+
+    def test_multiprocessing(self):
+        # Check caching works from multiple processes at once (#2028)
+        mod = self.import_module()
+        # Calling a pure Python caller of the JIT-compiled function is
+        # necessary to reproduce the issue.
+        f = mod.simple_usecase_caller
+        n = 3
+        pool = multiprocessing.Pool(n)
+        try:
+            res = sum(pool.imap(f, range(n)))
+        finally:
+            pool.close()
+        self.assertEqual(res, n * (n - 1) // 2)
+
+
 if __name__ == '__main__':
     unittest.main()
