@@ -10,8 +10,8 @@ import numpy as np
 
 import numba.unittest_support as unittest
 from numba.compiler import compile_isolated, Flags
-from numba import jit, types, typeinfer, utils, errors
-from numba.config import PYVERSION
+from numba import jit, numpy_support, types, typeinfer, utils, errors
+from numba.config import PYVERSION, MACHINE_BITS
 from .support import TestCase, tag
 from .true_div_usecase import truediv_usecase, itruediv_usecase
 from .matmul_usecase import (matmul_usecase, imatmul_usecase, DumbMatrix,
@@ -29,6 +29,22 @@ def make_static_power(exp):
     return pow_usecase
 
 
+# On 64-bit Windows, MSVC 2008 (Python 2.7) and Numpy 1.9, there's a
+# heisenbug that seems to manifest as wrong values returned by the CRT's
+# fmodf() function (which is called by LLVM's lowering of the `frem`
+# instruction).
+# Perhaps Numpy messing with some internal FP state that confuses the CRT?
+# Regardless, the only sane thing to do seems to be to skip the tests...
+
+_windows_floordiv_heisenbug = (
+    sys.platform == 'win32' and numpy_support.version[:2] == (1, 9)
+    and MACHINE_BITS == 64)
+
+_avoid_windows_floordiv_heisenbug = unittest.skipIf(
+    _windows_floordiv_heisenbug,
+    "avoid Windows + Numpy floordiv heisenbug - see PR #2057")
+
+
 class LiteralOperatorImpl(object):
 
     @staticmethod
@@ -415,8 +431,8 @@ class TestOperators(TestCase):
                 x_expected = copy.copy(x)
                 got = cfunc(x_got, y)
                 expected = pyfunc(x_expected, y)
-                self.assertTrue(np.allclose(got, expected))
-                self.assertTrue(np.allclose(x_got, x_expected))
+                np.testing.assert_allclose(got, expected, rtol=1e-5)
+                np.testing.assert_allclose(x_got, x_expected, rtol=1e-5)
 
     def coerce_operand(self, op, numba_type):
         if hasattr(op, "dtype"):
@@ -547,6 +563,10 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
+    @_avoid_windows_floordiv_heisenbug
+    def run_binop_floats_floordiv(self, pyfunc, flags=force_pyobj_flags):
+        self.run_binop_floats(pyfunc, flags=flags)
+
     def run_binop_complex(self, pyfunc, flags=force_pyobj_flags):
         x_operands = [-1.1 + 0.3j, 0.0 + 0.0j, 1.1j]
         y_operands = [-1.5 - 0.7j, 0.8j, 2.1 - 2.0j]
@@ -606,7 +626,7 @@ class TestOperators(TestCase):
     generate_binop_tests(locals(),
                          ('floordiv', 'ifloordiv', 'mod', 'imod'),
                          {'ints': 'run_binop_ints',
-                          'floats': 'run_binop_floats',
+                          'floats': 'run_binop_floats_floordiv',
                           })
 
     def check_div_errors(self, usecase_name, msg, flags=force_pyobj_flags,
@@ -1243,9 +1263,10 @@ class TestMixedInts(TestCase):
                 expected = control_func(x)
                 got = cfunc(x)
                 self.assertIsInstance(got, expected_type)
-                self.assertTrue(np.all(got == expected),
-                                "mismatch for %r with type %s: %r != %r"
-                                % (x, xt, got, expected))
+                self.assertPreciseEqual(
+                    got, expected,
+                    msg="mismatch for %r with type %s: %r != %r"
+                        % (x, xt, got, expected))
 
     def run_arith_binop(self, pyfunc, opname, samples,
                         expected_type=utils.INT_TYPES):
@@ -1432,7 +1453,7 @@ class TestStaticPower(TestCase):
 
     def test_real_values(self):
         exponents = [1, 2, 3, 5, 17, 0, -1, -2, -3, 0x111111, -0x111112]
-        vals = [1.5, 3.25, -1.25, np.float32(-1.5), float('inf'), float('nan')]
+        vals = [1.5, 3.25, -1.25, np.float32(-2.0), float('inf'), float('nan')]
 
         self._check_pow(exponents, vals)
 
