@@ -56,7 +56,7 @@ class TestTypingError(unittest.TestCase):
         try:
             compile_isolated(foo, ())
         except TypingError as e:
-            self.assertTrue(e.msg.startswith("Untyped global name"), e.msg)
+            self.assertIn("Untyped global name 'what'", str(e))
         else:
             self.fail("Should raise error")
 
@@ -64,7 +64,7 @@ class TestTypingError(unittest.TestCase):
         try:
             compile_isolated(bar, (types.int32,))
         except TypingError as e:
-            self.assertTrue(e.msg.startswith("Unknown attribute"), e.msg)
+            self.assertIn("Unknown attribute 'a' of type int32", str(e))
         else:
             self.fail("Should raise error")
 
@@ -154,5 +154,25 @@ class TestArgumentTypingError(unittest.TestCase):
         self.assertIn(expected, str(raises.exception))
 
 
+class TestCallError(unittest.TestCase):
+    def test_readonly_array(self):
+        @jit("(f8[:],)", nopython=True)
+        def inner(x):
+            return x
+
+        @jit(nopython=True)
+        def outer():
+            return inner(gvalues)
+
+        gvalues = np.ones(10, dtype=np.float64)
+
+        with self.assertRaises(TypingError) as raises:
+            outer()
+
+        got = str(raises.exception)
+        pat = r"Invalid usage of.*readonly array\(float64, 1d, C\)"
+        self.assertIsNotNone(re.search(pat, got))
+
+
 if __name__ == '__main__':
     unittest.main()
