@@ -12,23 +12,30 @@ import numba.unittest_support as unittest
 from numba import types, typing, utils, typeof, numpy_support, njit
 from numba.compiler import compile_isolated, Flags, DEFAULT_FLAGS
 from numba.numpy_support import from_dtype
-from numba import vectorize
+from numba import jit, vectorize
 from numba.config import PYVERSION
 from numba.errors import LoweringError, TypingError
 from .support import TestCase, CompilationCache, MemoryLeakMixin, tag
+from .test_operators import _avoid_windows_floordiv_heisenbug
 
 from numba.typing.npydecl import supported_ufuncs, all_ufuncs
 
 is32bits = tuple.__itemsize__ == 4
 iswindows = sys.platform.startswith('win32')
 
+# NOTE: to test the implementation of Numpy ufuncs, we disable rewriting
+# of array expressions.
+
 enable_pyobj_flags = Flags()
 enable_pyobj_flags.set("enable_pyobject")
+enable_pyobj_flags.set("no_rewrites")
 
 no_pyobj_flags = Flags()
+no_pyobj_flags.set("no_rewrites")
 
 enable_nrt_flags = Flags()
 enable_nrt_flags.set("nrt")
+enable_nrt_flags.set("no_rewrites")
 
 
 def _unimplemented(func):
@@ -217,30 +224,26 @@ class TestUFuncs(BaseUFuncTest, TestCase):
 
             cfunc(input_operand, result)
 
-            # Need special checks if NaNs are in results
-            if np.isnan(expected).any() or np.isnan(result).any():
-                self.assertTrue(np.allclose(np.isnan(result), np.isnan(expected)))
-                if not np.isnan(expected).all() and not np.isnan(result).all():
-                    self.assertTrue(np.allclose(result[np.invert(np.isnan(result))],
-                                     expected[np.invert(np.isnan(expected))]))
-            else:
-                match = np.all(result == expected) or np.allclose(result,
-                                                                  expected)
-                if not match:
-                    if invalid_flag:
-                        # Allow output to mismatch for invalid input
-                        print("Output mismatch for invalid input",
-                              input_tuple, result, expected)
-                    else:
-                        msg = '\n'.join(["ufunc '{0}' failed",
-                                         "inputs ({1}):", "{2}",
-                                         "got({3})", "{4}",
-                                         "expected ({5}):", "{6}"
-                                     ]).format(ufunc.__name__,
-                                               input_type, input_operand,
-                                               output_type, result,
-                                               expected.dtype, expected)
-                        self.fail(msg)
+            msg = '\n'.join(["ufunc '{0}' failed",
+                             "inputs ({1}):", "{2}",
+                             "got({3})", "{4}",
+                             "expected ({5}):", "{6}"
+                             ]).format(ufunc.__name__,
+                                       input_type, input_operand,
+                                       output_type, result,
+                                       expected.dtype, expected)
+
+            try:
+                np.testing.assert_array_almost_equal(expected, result,
+                                                     decimal=5,
+                                                     err_msg=msg)
+            except AssertionError:
+                if invalid_flag:
+                    # Allow output to mismatch for invalid input
+                    print("Output mismatch for invalid input",
+                          input_tuple, result, expected)
+                else:
+                    raise
 
     def binary_ufunc_test(self, ufunc, flags=no_pyobj_flags,
                          skip_inputs=[], additional_inputs=[],
@@ -333,8 +336,9 @@ class TestUFuncs(BaseUFuncTest, TestCase):
         self.binary_ufunc_test(np.true_divide, flags=flags, int_output_type=types.float64)
 
     @tag('important')
-    def test_floor_divide_ufunc(self):
-        self.binary_ufunc_test(np.floor_divide)
+    @_avoid_windows_floordiv_heisenbug
+    def test_floor_divide_ufunc(self, flags=no_pyobj_flags):
+        self.binary_ufunc_test(np.floor_divide, flags=flags)
 
     @tag('important')
     def test_negative_ufunc(self, flags=no_pyobj_flags):
@@ -349,6 +353,7 @@ class TestUFuncs(BaseUFuncTest, TestCase):
         self.binary_ufunc_test(np.power, flags=flags)
 
     @tag('important')
+    @_avoid_windows_floordiv_heisenbug
     def test_remainder_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.remainder, flags=flags)
 
@@ -654,28 +659,6 @@ class TestUFuncs(BaseUFuncTest, TestCase):
 
     ############################################################################
     # Other tests
-    def test_binary_ufunc_performance(self):
-
-        pyfunc = _make_binary_ufunc_usecase(np.add)
-        arraytype = types.Array(types.float32, 1, 'C')
-        cr = compile_isolated(pyfunc, (arraytype, arraytype, arraytype))
-        cfunc = cr.entry_point
-
-        nelem = 5000
-        x_operand = np.arange(nelem, dtype=np.float32)
-        y_operand = np.arange(nelem, dtype=np.float32)
-        control = np.empty_like(x_operand)
-        result = np.empty_like(x_operand)
-
-        def bm_python():
-            pyfunc(x_operand, y_operand, control)
-
-        def bm_numba():
-            cfunc(x_operand, y_operand, result)
-
-        print(utils.benchmark(bm_python, maxsec=.1))
-        print(utils.benchmark(bm_numba, maxsec=.1))
-        assert np.allclose(control, result)
 
     def binary_ufunc_mixed_types_test(self, ufunc, flags=no_pyobj_flags):
         ufunc_name = ufunc.__name__
@@ -744,15 +727,11 @@ class TestUFuncs(BaseUFuncTest, TestCase):
             cfunc(input1_operand, input2_operand, result)
             pyfunc(input1_operand, input2_operand, expected)
 
-            # Need special checks if NaNs are in results
-            if np.isnan(expected).any() or np.isnan(result).any():
-                self.assertTrue(np.allclose(np.isnan(result), np.isnan(expected)))
-                if not np.isnan(expected).all() and not np.isnan(result).all():
-                    self.assertTrue(np.allclose(result[np.invert(np.isnan(result))],
-                                     expected[np.invert(np.isnan(expected))]))
-            else:
-                self.assertTrue(np.all(result == expected) or
-                                np.allclose(result, expected))
+            scalar_type = getattr(output_type, 'dtype', output_type)
+            prec = ('single'
+                    if scalar_type in (types.float32, types.complex64)
+                    else 'double')
+            self.assertPreciseEqual(expected, result, prec=prec)
 
     def test_mixed_types(self):
         if not numpy_support.strict_ufunc_typing:
@@ -793,7 +772,7 @@ class TestUFuncs(BaseUFuncTest, TestCase):
             np.negative(x, expected)
 
             cfunc(x, result)
-            self.assertTrue(np.all(result == expected))
+            self.assertPreciseEqual(result, expected)
 
         # Test binary ufunc
         pyfunc = _make_binary_ufunc_usecase(np.add)
@@ -825,7 +804,7 @@ class TestUFuncs(BaseUFuncTest, TestCase):
             result = np.zeros(expected.shape, dtype='u8')
 
             cfunc(x, y, result)
-            self.assertTrue(np.all(result == expected))
+            self.assertPreciseEqual(result, expected)
 
     def test_implicit_output_npm(self):
         with self.assertRaises(TypeError):
@@ -1108,6 +1087,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
     def test_inplace_div(self):
         self.inplace_float_op_test('/=', [-1, 1.5, 3], [-5, 0, 2.5])
 
+    @_avoid_windows_floordiv_heisenbug
     def test_inplace_remainder(self):
         self.inplace_float_op_test('%=', [-1, 1.5, 3], [-5, 2, 2.5])
 
@@ -1176,6 +1156,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
         self.binary_op_test('/', int_output_type=int_out_type)
 
     @tag('important')
+    @_avoid_windows_floordiv_heisenbug
     def test_floor_divide_array_op(self):
         # Avoid floating-point zeros as x // 0.0 can have varying results
         # depending on the algorithm (which changed accross Numpy versions)
@@ -1203,6 +1184,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
         self.binary_op_test('//')
 
     @tag('important')
+    @_avoid_windows_floordiv_heisenbug
     def test_remainder_array_op(self):
         self.binary_op_test('%')
 
@@ -1362,7 +1344,7 @@ class TestUfuncIssues(TestCase):
 
         a = np.arange(10, dtype='f8')
         b = np.arange(10, dtype='f8')
-        self.assertTrue(np.all(foo(a, b) == (a + b) + (a + b)))
+        self.assertPreciseEqual(foo(a, b), (a + b) + (a + b))
 
     def test_issue_713(self):
         def foo(x,y):
@@ -1371,6 +1353,29 @@ class TestUfuncIssues(TestCase):
         cr = compile_isolated(foo, [types.complex128, types.complex128])
         self.assertEqual(foo(1j, 1j), cr.entry_point(1j, 1j))
 
+    def test_issue_2006(self):
+        """
+        <float32 ** int> should return float32, not float64.
+        """
+        def foo(x, y):
+            return np.power(x, y)
+        pyfunc = foo
+        cfunc = jit(nopython=True)(pyfunc)
+
+        def check(x, y):
+            got = cfunc(x, y)
+            np.testing.assert_array_almost_equal(got, pyfunc(x, y))
+            # Check the power operation conserved the input's dtype
+            # (this is different from Numpy, whose behaviour depends on
+            #  the *values* of the arguments -- see PyArray_CanCastArrayTo).
+            self.assertEqual(got.dtype, x.dtype)
+
+        xs = [np.float32([1, 2, 3]), np.complex64([1j, 2, 3-3j])]
+        for x in xs:
+            check(x, 3)
+            check(x, np.uint64(3))
+            check(x, np.int64([2, 2, 3]))
+
 
 class _LoopTypesTester(TestCase):
     """Test code generation for the different loop types defined by ufunc.
