@@ -220,7 +220,8 @@ def normalize_shape(shape):
             return types.UniTuple(dimtype, len(shape))
 
     elif isinstance(shape, types.Tuple) and shape.count == 0:
-        return shape
+        # Force (0 x intp) for consistency with other shapes
+        return types.UniTuple(types.intp, 0)
 
 
 @infer_getattr
@@ -352,6 +353,13 @@ class ArrayAttribute(AttributeTemplate):
         if ary.ndim == 1:
             return signature(types.none)
 
+    @bound_function("array.argsort")
+    def resolve_argsort(self, ary, args, kws):
+        assert not args
+        assert not kws
+        if ary.ndim == 1:
+            return signature(types.Array(types.intp, 1, 'C'))
+
     @bound_function("array.view")
     def resolve_view(self, ary, args, kws):
         from .npydecl import _parse_dtype
