@@ -10,10 +10,17 @@ import threading
 import timeit
 import math
 import sys
+import traceback
 
 import numpy as np
 
 from .six import *
+try:
+    # preferred over pure-python StringIO due to threadsafety
+    # note: parallel write to StringIO could cause data to go missing
+    from cStringIO import StringIO
+except ImportError:
+    from io import StringIO
 from numba.config import PYVERSION, MACHINE_BITS
 
 
@@ -26,6 +33,15 @@ if IS_PY3:
     get_ident = threading.get_ident
     intern = sys.intern
     file_replace = os.replace
+
+    def erase_traceback(exc_value):
+        """
+        Erase the traceback and hanging locals from the given exception instance.
+        """
+        if exc_value.__traceback__ is not None:
+            traceback.clear_frames(exc_value.__traceback__)
+        return exc_value.with_traceback(None)
+
 else:
     import thread
     import __builtin__ as builtins
@@ -44,6 +60,12 @@ else:
     else:
         file_replace = os.rename
 
+    def erase_traceback(exc_value):
+        """
+        Erase the traceback and hanging locals from the given exception instance.
+        """
+        return exc_value
+
 try:
     from inspect import signature as pysignature
 except ImportError:
@@ -144,7 +166,6 @@ def _at_shutdown():
     global _shutting_down
     _shutting_down = True
 
-atexit.register(_at_shutdown)
 
 def shutting_down(globals=globals):
     """
@@ -172,11 +193,21 @@ class ConfigOptions(object):
     def unset(self, name):
         self.set(name, False)
 
-    def __getattr__(self, name):
+    def _check_attr(self, name):
         if name not in self.OPTIONS:
-            raise NameError("Invalid flag: %s" % name)
+            raise AttributeError("Invalid flag: %s" % name)
+
+    def __getattr__(self, name):
+        self._check_attr(name)
         return self._values[name]
 
+    def __setattr__(self, name, value):
+        if name.startswith('_'):
+            super(ConfigOptions, self).__setattr__(name, value)
+        else:
+            self._check_attr(name)
+            self._values[name] = value
+
     def __repr__(self):
         return "Flags(%s)" % ', '.join('%s=%s' % (k, v)
                                        for k, v in self._values.items()
@@ -490,6 +521,20 @@ def total_ordering(cls):
     return cls
 
 
+def logger_hasHandlers(logger):
+    # Backport from python3.5 logging implementation of `.hasHandlers()`
+    c = logger
+    rv = False
+    while c:
+        if c.handlers:
+            rv = True
+            break
+        if not c.propagate:
+            break
+        else:
+            c = c.parent
+    return rv
+
 # Backported from Python 3.4: weakref.finalize()
 
 from weakref import ref
@@ -529,6 +574,7 @@ class finalize:
             import atexit
             atexit.register(self._exitfunc)
             finalize._registered_with_atexit = True
+            atexit.register(_at_shutdown)
         info = self._Info()
         info.weakref = ref(obj, self)
         info.func = func
@@ -629,3 +675,8 @@ class finalize:
             finalize._shutdown = True
             if reenable_gc:
                 gc.enable()
+
+
+# dummy invocation to force _at_shutdown() to be registered
+finalize(lambda: None, lambda: None)
+assert finalize._registered_with_atexit
